@@ -18,7 +18,8 @@ enum EVSFLogEntryType
   kVSFLogEntryDelete,
   kVSFLogEntryRename,
   kVSFLogEntryRmdir,
-  kVSFLogEntryChmod
+  kVSFLogEntryChmod,
+  kVSFLogEntryDebug,
 };
 
 /* vsf_log_init()
@@ -49,6 +50,12 @@ void vsf_log_start_entry(struct vsf_session* p_sess,
  */
 int vsf_log_entry_pending(struct vsf_session* p_sess);
 
+/* vsf_log_clear_entry()
+ * PURPOSE
+ * Clears any pending log entry.
+ */
+void vsf_log_clear_entry(struct vsf_session* p_sess);
+
 /* vsf_log_do_log()
  * PURPOSE
  * Denote the end of a logged operation, specifying whether the operation
