@@ -15,8 +15,7 @@ int ftp_write_str(const struct vsf_session* p_sess, const struct mystr* p_str,
 int ftp_read_data(struct vsf_session* p_sess, char* p_buf, unsigned int len);
 int ftp_write_data(const struct vsf_session* p_sess, const char* p_buf,
                    unsigned int len);
-void ftp_getline(const struct vsf_session* p_sess, struct mystr* p_str,
-                 char* p_buf);
+int ftp_getline(struct vsf_session* p_sess, struct mystr* p_str, char* p_buf);
 
 #endif /* VSF_READWRITE_H */
 
