@@ -21,7 +21,7 @@ void str_list_add(struct mystr_list* p_list, const struct mystr* p_str,
                   const struct mystr* p_sort_key_str);
 void str_list_sort(struct mystr_list* p_list, int reverse);
 
-int str_list_get_length(const struct mystr_list* p_list);
+unsigned int str_list_get_length(const struct mystr_list* p_list);
 int str_list_contains_str(const struct mystr_list* p_list,
                           const struct mystr* p_str);
 
